RollWindow=class(Turbine.UI.Lotro.Window)

function RollWindow:Constructor()
	Turbine.UI.Lotro.Window.Constructor(self);

	self:LoadSettings();

	--main container that cointains rows with rolls
	self.lbox = Turbine.UI.ListBox();
	self.lbox:SetParent(self);
	self.lbox:SetPosition(10,40);
	self.lbox:SetSize(190, 170);

	--reset button to clear all rolls
	self.bReset = Turbine.UI.Lotro.Button();
	self.bReset:SetParent(self);
	self.bReset:SetPosition(110,170);
	self.bReset:SetSize(80,20);
	self.bReset:SetText(_S[l][10]);

	self.bReset.Click = function(sender, args)
		self.lbox:ClearItems();
	end

	-- Give the listbox a scroll bar.
    local sbRolls = Turbine.UI.Lotro.ScrollBar();
    sbRolls:SetOrientation(Turbine.UI.Orientation.Vertical);
    sbRolls:SetParent(self);
    sbRolls:SetPosition(168, 38);
    sbRolls:SetSize(10, 209);
    
    self.lbox:SetVerticalScrollBar(sbRolls);

	self.KeyDown = function(sender, args)
		-- Hide window when pressing escape key.
		if (args.Action == Turbine.UI.Lotro.Action.Escape) then
			sender:SetVisible(false);
		end
	end

	self.PositionChanged = function(sender, args)
		local posX, posY = self:GetPosition();
		self.settings.posX = posX;
		self.settings.posY = posY;
		self:SaveSettings();
	end

	-- RollWindow settings
	self:SetSize(self.settings.width,self.settings.height);
	self:SetPosition(self.settings.posX, self.settings.posY);
	self:SetText(_S[l][1]); -- assigns the title bar text
	self:SetVisible(false); -- display the window (windows are not visible by default)
	self:SetResizable(true); -- needed for SizeChanged event
	self:SetMaximumSize(300,1200);
	self:SetMinimumSize(300,200);
	self:SetWantsKeyEvents(true); -- needed for KeyDown event

	local ChatMonitor = Turbine.Chat;

	ChatMonitor.resetStats = function ()
		ChatMonitor.myRoll = -1;
		ChatMonitor.winner = nil;
		ChatMonitor.winnerRoll = -1;
		ChatMonitor.itemData = nil;
		ChatMonitor.itemName = nil;
		ChatMonitor.legendary = false;
	end

	ChatMonitor.resetStats();

		

	--Turbine.Chat.Received: 
	-- 1) args.Message = contents of the message
	-- 2) args.Sender = not really working, only shows Player Character name
	-- 3) args.ChatType = ChatType from Turbine.ChatType
	-- 4) args.Error = shows instead of Sender on error messages
	ChatMonitor.Received = function(f, args)
		--for k, v in pairs(args) do
		--	Turbine.Shell.WriteLine(k .. ": " .. tostring(v));
		--end
		if (args.ChatType == Turbine.ChatType.Standard) then

			-- If chat message contains keyword rolled
			if (args.Message:find(_S[l][5])) then  

				-- retreive ItemInfo from the first line of rolls
				if (ChatMonitor.itemData == nil) then
					ChatMonitor.itemData, ChatMonitor.itemName = args.Message:match("<ExamineItemInstance:ItemInfo:(.-)>(%b[])<\\ExamineItemInstance>");
					if (not ChatMonitor.itemData) then
						ChatMonitor.itemData, ChatMonitor.itemName = args.Message:match("<ExamineIA:IAInfo:(.-)>(%b[])<\\ExamineIA>");
						ChatMonitor.legendary = true;
					end
					--clear lines if there got something from manual rolls?
					--if ChatMonitor.itemData then ChatMonitor.resetStats(); end;
					--Lib.print("item found: " .. ChatMonitor.itemData .. " - " .. ChatMonitor.itemName);
				end

				-- If it is my roll (found with keyword "You rolled"), save my roll (and set me up as winner if winning)
				if (args.Message:find(_S[l][6])) then
					ChatMonitor.myRoll = args.Message:match(_S[l][8]);
					--Lib.print("myRoll: " .. tostring(ChatMonitor.myRoll));
					if tonumber(ChatMonitor.myRoll) > ChatMonitor.winnerRoll then
						ChatMonitor.winner= Turbine.Gameplay.LocalPlayer.GetInstance():GetName();
						ChatMonitor.winnerRoll=tonumber(ChatMonitor.myRoll);
						--Lib.print("winner updated");
					end

				-- if it is another player's roll, compare the roll against winner and save his name and roll if winning
				else 
					local player, playerRoll = args.Message:match(_S[l][9]);
					--Lib.print(player .. " rolls: " .. playerRoll .. " " .. tostring(ChatMonitor.winnerRoll));
					if tonumber(playerRoll) > ChatMonitor.winnerRoll then
						ChatMonitor.winner=player:sub(2,-2);
						ChatMonitor.winnerRoll=tonumber(playerRoll);
						--Lib.print("winner updated");
					end
				end
			end

			-- when we parse the last line announcing winner (keyword "win"), display the rolled item in RollViewer window and reset stats
			-- when there is only one player rolling, skip it
			if (args.Message:find(_S[l][7]) and ChatMonitor.itemData) then
				local itemDecoded = Lib.Decode.DecodeLinkData(ChatMonitor.itemData, ChatMonitor.legendary);
				self.lbox:AddItem(RollControl(Lib.NewItemInfo(itemDecoded.itemGID), ChatMonitor.winner, ChatMonitor.winnerRoll, ChatMonitor.myRoll));
				--Lib.print(ChatMonitor.winner.." wins "..ChatMonitor.itemName.." with a roll of "..ChatMonitor.winnerRoll);
				ChatMonitor.resetStats();
			end
		end

	end

end

function RollWindow:SizeChanged(args)
	self.settings.width, self.settings.height = self:GetSize();
	self.lbox:SetSize(self.settings.width-15,self.settings.height-80);
	self.bReset:SetPosition(self.settings.width/2-40, self.settings.height-30);
	self.lbox:GetVerticalScrollBar():SetPosition(self.settings.width-20, 30);
	self.lbox:GetVerticalScrollBar():SetSize(10, self.settings.height-60);
	
	self:SaveSettings();
end

function RollWindow:LoadSettings()
	-- Load settings.  Use default values if not set.
	self.settings = Lib.PatchDataLoad(Turbine.DataScope.Character, "RollViewer");
	if (type(self.settings) ~= "table") then
		self.settings = { };
	end
	if not self.settings.width then
		self.settings.width = 300
	end
	if not self.settings.height then
		self.settings.height = 200
	end
	if not self.settings.posX then
		self.settings.posX = Turbine.UI.Display:GetWidth()/2-self.settings.width;
	end
	
	if not self.settings.posY then
		self.settings.posY = Turbine.UI.Display:GetHeight()/2-self.settings.height;
	end
end

function RollWindow:SaveSettings()
	Lib.PatchDataSave(Turbine.DataScope.Character, "RollViewer", self.settings);
end